@@ -1,16 +1,16 @@
 package Text::ANSI::Util;
 
-our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
-our $DATE = '2021-04-14'; # DATE
-our $DIST = 'Text-ANSI-Util'; # DIST
-our $VERSION = '0.232'; # VERSION
-
 use 5.010001;
 use strict 'subs', 'vars';
 use warnings;
 
-require Exporter;
-our @ISA       = qw(Exporter);
+use Exporter qw(import);
+
+our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
+our $DATE = '2023-02-27'; # DATE
+our $DIST = 'Text-ANSI-Util'; # DIST
+our $VERSION = '0.234'; # VERSION
+
 our @EXPORT_OK = qw(
                        ta_add_color_resets
                        ta_detect
@@ -48,7 +48,7 @@ Text::ANSI::Util - Routines for text containing ANSI color codes
 
 =head1 VERSION
 
-This document describes version 0.232 of Text::ANSI::Util (from Perl distribution Text-ANSI-Util), released on 2021-04-14.
+This document describes version 0.234 of Text::ANSI::Util (from Perl distribution Text-ANSI-Util), released on 2023-02-27.
 
 =head1 SYNOPSIS
 
@@ -294,16 +294,16 @@ Options:
 
 =item * flindent => STR
 
-First line indent. See Text::WideChar::Util for more details.
+First line indent. Currently must not contain ANSI color codes or wide
+characters.
 
 =item * slindent => STR
 
-First line indent. See Text::WideChar::Util for more details.
+Subsequent line indent. Currently must not contain ANSI color codes or wide
+characters.
 
 =item * tab_width => INT (default: 8)
 
-First line indent. See Text::WideChar::Util for more details.
-
 =item * pad => BOOL (default: 0)
 
 If set to true, will pad each line to C<$width>. This is convenient if you need
@@ -374,14 +374,6 @@ Please visit the project's homepage at L<https://metacpan.org/release/Text-ANSI-
 
 Source repository is at L<https://github.com/perlancar/perl-Text-ANSI-Util>.
 
-=head1 BUGS
-
-Please report any bugs or feature requests on the bugtracker website L<https://github.com/perlancar/perl-Text-ANSI-Util/issues>
-
-When submitting a bug or request, please include a test-file or a
-patch to an existing test-file that illustrates the bug or desired
-feature.
-
 =head1 SEE ALSO
 
 L<String::Pad> provides padding function for strings that do not contain ASCII
@@ -400,11 +392,43 @@ codes I<and> wide characters.
 
 perlancar <perlancar@cpan.org>
 
+=head1 CONTRIBUTOR
+
+=for stopwords Steven Haryanto
+
+Steven Haryanto <stevenharyanto@gmail.com>
+
+=head1 CONTRIBUTING
+
+
+To contribute, you can send patches by email/via RT, or send pull requests on
+GitHub.
+
+Most of the time, you don't need to build the distribution yourself. You can
+simply modify the code, then test via:
+
+ % prove -l
+
+If you want to build the distribution (e.g. to try to install it locally on your
+system), you can install L<Dist::Zilla>,
+L<Dist::Zilla::PluginBundle::Author::PERLANCAR>,
+L<Pod::Weaver::PluginBundle::Author::PERLANCAR>, and sometimes one or two other
+Dist::Zilla- and/or Pod::Weaver plugins. Any additional steps required beyond
+that are considered a bug and can be reported to me.
+
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2021, 2016, 2015, 2014, 2013 by perlancar@cpan.org.
+This software is copyright (c) 2023, 2021, 2016, 2015, 2014, 2013 by perlancar <perlancar@cpan.org>.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
 
+=head1 BUGS
+
+Please report any bugs or feature requests on the bugtracker website L<https://rt.cpan.org/Public/Dist/Display.html?Name=Text-ANSI-Util>
+
+When submitting a bug or request, please include a test-file or a
+patch to an existing test-file that illustrates the bug or desired
+feature.
+
 =cut
